package org.apithefire.web.pathmap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apithefire.util.Objects;

public class PathMap {

	private static class Pair {
		
		public final PathPattern pattern;
		public final Object object;
		
		public Pair(PathPattern pathPattern, Object object) {
			this.pattern = Objects.nonNull(pathPattern);
			this.object = Objects.nonNull(object);
		}
		
	}
	
	private final Map<ExactPathPattern, Object> exactMap;
	private final SortedMap<PrefixPathPattern, Object> prefixMap;
	private final Map<String, Object> suffixMap;
	private final List<Pair> defaults;
	private final List<Pair> all;
	
	public PathMap() {
		exactMap = new HashMap<ExactPathPattern, Object>();
		prefixMap = new TreeMap<PrefixPathPattern, Object>();
		suffixMap = new HashMap<String, Object>();
		defaults = new ArrayList<Pair>();
		all = new ArrayList<Pair>();
	}
	
	public void put(String patternStr, Object object) {
		Objects.nonNull(object);
		
		PathPattern pattern = PathPattern.parse(patternStr);
		
		if (pattern instanceof ExactPathPattern) {
			exactMap.put((ExactPathPattern) pattern, object);
		} else if (pattern instanceof PrefixPathPattern) {
			prefixMap.put((PrefixPathPattern) pattern, object);
		} else if (pattern instanceof SuffixPathPattern) {
			SuffixPathPattern suffixPattern = (SuffixPathPattern) pattern;
			suffixMap.put(suffixPattern.getSuffix(), object);
		} else if (pattern == PathPattern.defaultPathPattern) {
			defaults.add(new Pair(pattern, object));
		} else {
			throw new AssertionError();
		}
		all.add(new Pair(pattern, object));
	}
	
	public Object get(String path) {
		if (!exactMap.isEmpty()) {
			for (Map.Entry<ExactPathPattern, Object> entry : exactMap.entrySet()) {
				ExactPathPattern pattern = entry.getKey();
				
				if (pattern.matches(path)) {
					return entry.getValue();
				}
			}
		}
		if (!prefixMap.isEmpty()) {
			for (Map.Entry<PrefixPathPattern, Object> entry : prefixMap.entrySet()) {
				PrefixPathPattern pattern = entry.getKey();
				
				if (pattern.matches(path)) {
					return entry.getValue();
				}
			}
		}
		if (!suffixMap.isEmpty()) {
			String suffix = PathPattern.getSuffix(path);
			Object object = suffixMap.get(suffix);
			
			if (object != null) {
				return object;
			}
		}
		if (!defaults.isEmpty()) {
			return defaults.get(0).object;
		}
		return null;
	}
	
}
