package org.apithefire.web.pathmap;

abstract class PathPattern {
	
	protected static final PathPattern defaultPathPattern = new PathPattern() {
		
		@Override
		public boolean matches(String path) {
			return true;
		}
		
	};

	protected static final String SLASH = "/";
	protected static final String DOT = ".";
	
	public static PathPattern parse(String pattern) {
		if (pattern.length() == 0) {
			throw new IllegalArgumentException("Path pattern is empty.");
		}
		if (pattern.startsWith(SLASH) && pattern.endsWith("/*")) {
			String prefix = pattern.substring(0, pattern.length() - 1);
			return new PrefixPathPattern(prefix);
		}
		if (pattern.startsWith("*.") && pattern.length() > 2) {
			return new SuffixPathPattern(getSuffix(pattern));
		}
		if (pattern.equals(SLASH)) {
			return defaultPathPattern;
		}
		return new ExactPathPattern(pattern);
	}
	
	public abstract boolean matches(String path);
	
	protected static String fixPath(String path) {
		if (path.length() == 0) {
			return SLASH;
		}
		
		boolean start = path.startsWith(SLASH);
		boolean end = path.endsWith(SLASH);
		
		if (!start && !end) {
			return SLASH + path + SLASH;
		}
		if (!start) {
			return SLASH + path;
		}
		if (!end) {
			return path + SLASH;
		}
		return path;
	}
	
	protected static String getSuffix(String path) {
		int dotIndex = path.lastIndexOf(DOT);
		
		if (dotIndex == -1) {
			return null;
		}
		return path.substring(dotIndex + 1);
	}
	
}
