package org.apithefire.web.pathmap;

class PrefixPathPattern extends PathPattern
		implements Comparable<PrefixPathPattern> {

	private static final char SLASH_CHAR = SLASH.charAt(0);
	
	private String prefix;
	private int componentCount;
	
	public PrefixPathPattern(String prefix) {
		this.prefix = fixPath(prefix);
		this.componentCount = countComponents(prefix);
	}
	
	protected static int countComponents(String path) {
		if (path.length() == 0 || path.equals(SLASH)) {
			return 0;
		}
		path = fixPath(path);
		
		int slashCount = 0;
		int pathLength = path.length();
		
		for (int i = 0; i < pathLength; ++i) {
			if (path.charAt(i) == SLASH_CHAR) {
				++slashCount;
			}
		}
		return slashCount - 1;
	}
	
	public String getPrefix() {
		return prefix;
	}
	
	public int getComponentCount() {
		return componentCount;
	}

	@Override
	public boolean matches(String path) {
		return fixPath(path).startsWith(prefix);
	}
	
	public int compareTo(PrefixPathPattern pathPattern) {
		if (componentCount != pathPattern.componentCount) {
			return pathPattern.componentCount - componentCount;
		}
		return pathPattern.prefix.compareTo(prefix);
	}
	
}